/*
 *	Real Time Clock Alarm Setting Program
 *
 *	Compile with:
 *		gcc -s -Wall -Wstrict-prototypes rtc.c -o rtc
 *
 *      Modified from the example program by:
 *	Copyright (C) 1996, Paul Gortmaker
 *	for use on a Psion 5MX.
 * 
 *      May 2006
 *
 *	Released under the GNU General Public License, version 2,
 *	included herein by reference.
 *
 */

#include <stdio.h>
#include <linux/rtc.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

void usage() {
  printf("Usage:\n\n"
         "rtc [+]hh:mm\n\n"
         "Example:\n"
         "   rtc +1:15    will set alarm for one hour and 15 minutes\n"
         "   rtc 8:00     will set alarm for 8:00 am\n\n"
         "Bugs: Daylight saving jumps are not taken into account.\n"
         "      Be sure RTC and system clocks are set to the same local or UTC time.\n"
         "      Modem ring detection may trigger early wake-up.\n"
         "                                                     \n");
}

int main(int argc, char **argv) {

int fd, retval, irqcount = 0;
unsigned long data;
struct rtc_time rtc_tm;
int isoffset;       /* setting time offset */
int hour, minute;   /* alarm time requested */
int hour1, minute1;  /* backup for precise_mode */
char *p;

if (optind < argc) {
    if (optind+1 != argc) {
      fprintf(stderr,"To many arguments.\n"); usage();
      exit(1);
    }

p=argv[optind];
  isoffset=0;
  if (p[0]=='+') { p++; isoffset=1;}
  if (2!=sscanf(p, "%d:%d", &hour, &minute)) {
    fprintf(stderr,"rtc: Bad argument(s)\n");
    exit(1);
}

hour1=hour; minute1=minute; /* backup */

  } else {
    fprintf(stderr,"rtc: missing argument.\n"); usage();
    exit(1);
  }


  /* argument processing finished */

if (geteuid()!=0) {
   fprintf(stderr, "rtc: warning: This program must be run as root "
              "or have the SUID attribute set\n");
}


fd = open ("/dev/rtc", O_RDONLY);

if (fd ==  -1) {
	perror("/dev/rtc");
	exit(errno);
}

fprintf(stderr, "RTC Alarm Set.\n");

/* Read the RTC time/date */

retval = ioctl(fd, RTC_RD_TIME, &rtc_tm);
if (retval == -1) {
	perror("ioctl");
	exit(errno);
}

fprintf(stderr, "Current RTC date/time is %d-%d-%d, %02d:%02d:%02d.\n",
	rtc_tm.tm_mday, rtc_tm.tm_mon + 1, rtc_tm.tm_year + 1900,
	rtc_tm.tm_hour, rtc_tm.tm_min, rtc_tm.tm_sec);

/* Set the alarm - first is offset case */
  if (isoffset) {
    if (hour == 0 && minute == 0)
      exit(0);
    hour+=(int)rtc_tm.tm_hour;
    minute+=(int)rtc_tm.tm_min;
     } 

/* subtract a minute off...see below */
    minute-=1;

    while (minute<0) {minute +=60; hour--; }
    while (minute>=60) {minute -=60; hour++; }
    while (hour < 0) hour += 24;
    hour %= 24;

    rtc_tm.tm_min = minute;
    rtc_tm.tm_hour = hour;
/* The second time is irrelevant - so set it to 55 s */
/* and subtract 1 from the minute to make the midnight */
/* barrier jump shorter. */
    rtc_tm.tm_sec = 55;

retval = ioctl(fd, RTC_ALM_SET, &rtc_tm);
if (retval == -1) {
	perror("ioctl");
	exit(errno);
}

/* Read the current alarm settings */
retval = ioctl(fd, RTC_ALM_READ, &rtc_tm);
if (retval == -1) {
	perror("ioctl");
	exit(errno);
}

fprintf(stderr, "Alarm time now set to %02d:%02d:%02d.\n",
	rtc_tm.tm_hour, rtc_tm.tm_min, rtc_tm.tm_sec);
fprintf(stderr, "On %02d/%02d/%04d.\n",
	rtc_tm.tm_mday, rtc_tm.tm_mon+1, rtc_tm.tm_year+1900);

/* Enable alarm interrupts */
retval = ioctl(fd, RTC_AIE_ON, 0);
if (retval == -1) {
	perror("ioctl");
	exit(errno);
}

fprintf(stderr, "Waiting for alarm...\n");
fflush(stderr);
/* This blocks until the alarm ring causes an interrupt */
retval = read(fd, &data, sizeof(unsigned long));
if (retval == -1) {
	perror("read");
	exit(errno);
}
irqcount++;
fprintf(stderr, "Okay. Alarm rang.\n");

/* Disable alarm interrupts */
retval = ioctl(fd, RTC_AIE_OFF, 0);
if (retval == -1) {
	perror("ioctl");
	exit(errno);
}


fprintf(stderr, "Alarm complete.\n");

close(fd);
return 0;

} /* end main */
